/*
 * classicMOP.c
 *
 *  Created on: 26/10/2010
 *      Author: Saúl Zapotecas
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include "../global.h"
#include "../io_data.h"

/* Fonseca - MOP [-4, 4] */
void Fonseca(double *xreal, double *obj, double *constr)
{
	double s1, s2;
	int i;
	double n = (double) mop.nreal;
	s1 = s2 = 0.0;
	for (i = 0; i < mop.nreal; i++)
	{
		s1 += pow((xreal[i] - (1.0 / sqrt(n))), 2.0);
		s2 += pow((xreal[i] + (1.0 / sqrt(n))), 2.0);
	}
	obj[0] = 1.0 - exp(-s1);
	obj[1] = 1.0 - exp(-s2);
	return;
}

/* Kursawe - MOP [-5,5] */
void Kursawe(double *xreal, double *obj, double *constr)
{
	int i;
	double a = 0.8;
	double b = 3.0;

	obj[0] = obj[1] = 0.0;
	for (i = 0; i < mop.nreal - 1; i++)
	{
		obj[0] += -10.0
				* exp(-0.2 * sqrt(pow(xreal[i], 2.0) + pow(xreal[i + 1], 2.0)));

	}
	for (i = 0; i < mop.nreal; i++)
	{
		obj[1] += pow(fabs(xreal[i]), a) + 5.0 * sin(pow(xreal[i], b));
	}
	return;
}

/* Poli - MOP (2vars, 2obj) */
void Poli(double *xreal, double *obj, double *constr)
{
	double a1, a2, b1, b2;
	a1 = 0.5 * sin(1.0) - 2.0 * cos(1.0) + sin(2.0) - 1.5 * cos(2.0);
	a2 = 1.5 * sin(1.0) - cos(1.0) + 2.0 * sin(2.0) - 0.5 * cos(2.0);
	b1 = 0.5 * sin(xreal[0]) - 2.0 * cos(xreal[0]) + sin(xreal[1])
			- 1.5 * cos(xreal[1]);
	b2 = 1.5 * sin(xreal[0]) - cos(xreal[0]) + 2.0 * sin(xreal[1])
			- 0.5 * cos(xreal[1]);
	obj[0] = 1.0 + pow((a1 - b1), 2.0) + pow((a2 - b2), 2.0);
	obj[1] = pow((xreal[0] + 3.0), 2.0) + pow((xreal[1] + 1.0), 2.0);
	return;
}

/* Schaffer - MOP (1var, 2obj) [-1000, 1000] */
void Schaffer(double *xreal, double *obj, double *constr)
{
	obj[0] = xreal[0] * xreal[0];
	obj[1] = (xreal[0] - 2.0) * (xreal[0] - 2.0);
	return;
}

/* Viennet1 - MOP (2vars, 3obj) [-2,2]*/
void Viennet1(double *xreal, double *obj, double *constr)
{
	obj[0] = pow(xreal[0], 2.0) + pow(xreal[1] - 1.0, 2.0);
	obj[1] = pow(xreal[0], 2.0) + pow(xreal[1] + 1.0, 2.0) + 1.0;
	obj[2] = pow(xreal[0] - 1.0, 2.0) + pow(xreal[1], 2.0) + 2.0;
	return;
}

/* Viennet2 - MOP (2vars, 3obj) [-4,4]*/
void Viennet2(double *xreal, double *obj, double *constr)
{
	obj[0] = pow((xreal[0] - 2.0), 2.0) / 2.0
			+ pow((xreal[1] + 1.0), 2.0) / 13.0 + 3.0;
	obj[1] = pow((xreal[0] + xreal[1] - 3.0), 2.0) / 36.0
			+ pow((-xreal[0] + xreal[1] + 2.0), 2.0) / 8.0 - 17.0;
	obj[2] = pow((xreal[0] + 2.0 * xreal[1] - 1.0), 2.0) / 175.0
			+ pow((2.0 * xreal[1] - xreal[0]), 2.0) / 17.0 - 13.0;
	return;
}

/* Viennet3 - MOP (2vars, 3obj) [-3, 3] */
void Viennet3(double *xreal, double *obj, double *constr)
{
	double value1 = 3.0 * xreal[0] - 2.0 * xreal[1] + 4.0;
	double value2 = xreal[0] - xreal[1] + 1.0;

	obj[0] = 0.5 * (pow(xreal[0], 2.0) + pow(xreal[1], 2.0))
			+ sin(pow(xreal[0], 2.0) + pow(xreal[1], 2.0));

	obj[1] = pow(value1, 2.0) / 8.0 + pow(value2, 2.0) / 27.0 + 15.0;
	obj[2] = 1.0 / (pow(xreal[0], 2.0) + pow(xreal[1], 2.0) + 1.0)
			- 1.1 * exp(-pow(xreal[0], 2.0) - pow(xreal[1], 2.0));
	return;
}
