/*
 * DTLZs.c
 *
 *  Created on: 10/02/2010
 *      Author: Saúl Zapotecas
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include "../global.h"
#include "../io_data.h"

/* DTLZ1 - MOP */
void DTLZ1(double *xreal, double *obj, double *constr)
{
	int i;
	double sum;
	double g;
	int n = mop.nreal;
	int m = mop.nobj;
	int k = n - m + 1;

	sum = 0.0;
	for (i = m - 1; i < n; i++)
	{
		sum += pow(xreal[i] - 0.5, 2.0) - cos(20.0 * PI * (xreal[i] - 0.5));
	}
	g = 100.0 * (k + sum);

	obj[0] = 0.5 * (1.0 + g) * xreal[0] * xreal[1];
	obj[1] = 0.5 * (1.0 + g) * (1.0 - xreal[1]) * xreal[0];
	obj[2] = 0.5 * (1.0 + g) * (1.0 - xreal[0]);
	return;
}

/* DTLZ2 - MOP */
void DTLZ2(double *xreal, double *obj, double *constr)
{
	int i;
	double sum;
	double g;
	int n = mop.nreal;
	int m = mop.nobj;

	sum = 0.0;
	for (i = m - 1; i < n; i++)
	{
		sum += pow(xreal[i] - 0.5, 2.0);
	}
	g = sum;

	obj[0] = (1.0 + g) * cos(xreal[0] * PI * 0.5) * cos(xreal[1] * PI * 0.5);
	obj[1] = (1.0 + g) * cos(xreal[0] * PI * 0.5) * sin(xreal[1] * PI * 0.5);
	obj[2] = (1.0 + g) * sin(xreal[0] * PI * 0.5);
	return;
}

/* DTLZ3 - MOP */
void DTLZ3(double *xreal, double *obj, double *constr)
{
	int i;
	double sum;
	double g;
	int n = mop.nreal;
	int m = mop.nobj;
	int k = n - m + 1;

	sum = 0.0;
	for (i = m - 1; i < n; i++)
	{
		sum += pow(xreal[i] - 0.5, 2.0) - cos(20.0 * PI * (xreal[i] - 0.5));
	}
	g = 100.0 * (k + sum);

	obj[0] = (1.0 + g) * cos(xreal[0] * PI * 0.5) * cos(xreal[1] * PI * 0.5);
	obj[1] = (1.0 + g) * cos(xreal[0] * PI * 0.5) * sin(xreal[1] * PI * 0.5);
	obj[2] = (1.0 + g) * sin(xreal[0] * PI * 0.5);
	return;
}

/* DTLZ4 - MOP */
void DTLZ4(double *xreal, double *obj, double *constr)
{
	int i;
	double alpha;
	double g, factor1, factor2;
	int n = mop.nreal;
	int m = mop.nobj;

	alpha = PI;
	g = 0.0;
	for (i = m - 1; i < n; i++)
	{
		g += pow(xreal[i] - 0.5, 2.0);
	}

	factor1 = pow(xreal[0], alpha);
	factor2 = pow(xreal[1], alpha);
	obj[0] = (1.0 + g) * cos(factor1 * PI * 0.5) * cos(factor2 * PI * 0.5);
	obj[1] = (1.0 + g) * cos(factor1 * PI * 0.5) * sin(factor2 * PI * 0.5);
	obj[2] = (1.0 + g) * sin(factor1 * PI * 0.5);
	return;
}

/* DTLZ5 - MOP */
void DTLZ5(double *xreal, double *obj, double *constr)
{
	int i;
	double g;
	double tetha[3];
	int n = mop.nreal;
	int m = mop.nobj;

	g = 0.0;
	for (i = m - 1; i < n; i++)
	{
		g += pow(xreal[i] - 0.5, 2.0);
	}

	tetha[0] = (PI * 0.5) * xreal[0];
	tetha[1] = (PI / (4.0 * (1.0 + g))) * (1.0 + 2.0 * g * xreal[1]);

	obj[0] = (1.0 + g) * cos(tetha[0]) * cos(tetha[1]);
	obj[1] = (1.0 + g) * cos(tetha[0]) * sin(tetha[1]);
	obj[2] = (1.0 + g) * sin(tetha[0]);
	return;
}

/* DTLZ6 - MOP */
void DTLZ6(double *xreal, double *obj, double *constr)
{
	int i;
	double g;
	double tetha[3];
	int n = mop.nreal;
	int m = mop.nobj;

	g = 0.0;
	for (i = m - 1; i < n; i++)
	{
		g += pow(xreal[i], 0.1);
	}

	tetha[0] = (PI * 0.5) * xreal[0];
	tetha[1] = (PI / (4.0 * (1.0 + g))) * (1.0 + 2.0 * g * xreal[1]);
	obj[0] = (1.0 + g) * cos(tetha[0]) * cos(tetha[1]);
	obj[1] = (1.0 + g) * cos(tetha[0]) * sin(tetha[1]);
	obj[2] = (1.0 + g) * sin(tetha[0]);
	return;
}

/* DTLZ7 - MOP */
void DTLZ7(double *xreal, double *obj, double *constr)
{
	int i;
	double sum;
	double g, h;
	int n = mop.nreal;
	int m = mop.nobj;
	int k = n - m + 1;

	sum = 0.0;
	for (i = m - 1; i < n; i++)
	{
		sum += xreal[i];
	}
	g = 1.0 + (9.0 / (double) k) * sum;

	obj[0] = xreal[0];
	obj[1] = xreal[1];
	h = m - (((obj[0] / (1.0 + g)) * (1.0 + sin(3.0 * PI * obj[0]))) + ((obj[1]
			/ (1.0 + g)) * (1.0 + sin(3.0 * PI * obj[1]))));
	obj[2] = (1.0 + g) * h;
	return;
}
