/*
 * ZDTs.c
 *
 *  Created on: 10/02/2010
 *      Author: Saúl Zapotecas
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include "../global.h"
#include "../io_data.h"

/* ZDT1 - MOP */
void ZDT1(double *xreal, double *obj, double *constr)
{
	double f1, f2, g, h, sum;
	int i;

	f1 = xreal[0];
	sum = 0.0;
	for (i = 1; i < mop.nreal; i++)
	{
		sum += xreal[i];
	}
	g = 1.0 + 9.0 * sum / (mop.nreal - 1.0);
	h = 1.0 - sqrt(f1 / g);
	f2 = g * h;
	obj[0] = f1;
	obj[1] = f2;
	return;
}

/* ZDT2 - MOP */
void ZDT2(double *xreal, double *obj, double *constr)
{
	double f1, f2, g, h, sum;
	int i;

	f1 = xreal[0];
	sum = 0.0;
	for (i = 1; i < mop.nreal; i++)
	{
		sum += xreal[i];
	}
	g = 1.0 + 9.0 * sum / (mop.nreal - 1.0);
	h = 1.0 - pow((f1 / g), 2.0);
	f2 = g * h;
	obj[0] = f1;
	obj[1] = f2;
	return;
}

/* ZDT3 - MOP */
void ZDT3(double *xreal, double *obj, double *constr)
{
	double f1, f2, g, h, sum;
	int i;

	f1 = xreal[0];
	sum = 0.0;
	for (i = 1; i < mop.nreal; i++)
	{
		sum += xreal[i];
	}
	g = 1.0 + 9.0 * sum / (mop.nreal - 1.0);
	h = 1.0 - sqrt(f1 / g) - (f1 / g) * sin(10.0 * PI * f1);
	f2 = g * h;
	obj[0] = f1;
	obj[1] = f2;
	return;
}

/* ZDT4 - MOP */
void ZDT4(double *xreal, double *obj, double *constr)
{
	double f1, f2, g, h, sum;
	int i;

	f1 = xreal[0];
	sum = 0.0;
	for (i = 1; i < mop.nreal; i++)
	{
		sum += (pow(xreal[i], 2.0) - 10.0 * cos(4.0 * PI * xreal[i]));
	}
	g = 1.0 + 10.0 * (mop.nreal - 1.0) + sum;
	h = 1.0 - sqrt(f1 / g);
	f2 = g * h;
	obj[0] = f1;
	obj[1] = f2;
	return;
}

/* ZDT6 - MOP */
void ZDT6(double *xreal, double *obj, double *constr)
{
	double f1, f2, g, h, sum;
	int i;

	f1 = 1.0 - exp(-4.0 * xreal[0]) * pow(sin(6.0 * PI * xreal[0]), 6.0);
	sum = 0.0;
	for (i = 1; i < mop.nreal; i++)
	{
		sum += xreal[i];
	}
	g = 1.0 + 9.0 * pow((sum / (mop.nreal - 1.0)), 0.25);
	h = 1.0 - pow((f1 / g), 2.0);
	f2 = g * h;
	obj[0] = f1;
	obj[1] = f2;
	return;
}
