/*
 * allocate.c
 *
 *  Created on: 11/02/2010
 *      Author: Saúl Zapotecas
 */

#include <stdio.h>
#include <stdlib.h>

#include "global.h"

/* *
 * Allocate memory of an individual
 * */
void allocate_memory_ind(individual *ind)
{
	ind->xreal = (double*) malloc(mop.nreal * sizeof(double));
	ind->obj = (double*) malloc(mop.nobj * sizeof(double));
	if (mop.nconst > 0)
	{
		ind->constr = (double*) malloc(mop.nconst * sizeof(double));
	}

	ind->vel = (double*) malloc(mop.nreal * sizeof(double));
	ind->pb_xreal = (double*) malloc(mop.nreal * sizeof(double));
	ind->pb_obj = (double*) malloc(mop.nobj * sizeof(double));
	return;
}

/* *
 * Deallocate memory of an individual
 * */
void deallocate_memory_ind(individual *ind)
{
	free(ind->xreal);
	free(ind->obj);
	if (mop.nconst > 0)
	{
		free(ind->constr);
	}
	free(ind->vel);
	free(ind->pb_xreal);
	free(ind->pb_obj);
	return;
}

/* *********************************************
 * Allocate memory of a population
 * *********************************************/
void allocate_memory_pop(population *pop, int size)
{
	int i;
	pop->size = size;
	pop->ind = (individual*) malloc(pop->size * sizeof(individual));
	for (i = 0; i < pop->size; i++)
	{
		allocate_memory_ind(&(pop->ind[i]));
	}
	return;
}

void allocate_memory_vector(vector *v)
{
	v->w = (double*) malloc(sizeof(double) * mop.nobj);
	v->aux = -1;
	return;
}

void deallocate_memory_vector(vector *v)
{
	free(v->w);
	return;
}

/* *********************************************
 * Allocate memory of a set of weight
 * *********************************************/
void allocate_memory_weight(weight *w, int size)
{
	int i;
	w->size = size;
	w->weight = (vector*) malloc(w->size * sizeof(vector));
	for (i = 0; i < w->size; i++)
	{
		allocate_memory_vector(&(w->weight[i]));
	}
	return;
}

void deallocate_memory_weight(weight *w)
{
	int i;

	for (i = 0; i < w->size; i++)
	{
		deallocate_memory_vector(&(w->weight[i]));
	}
	free(w->weight);
	return;
}

void deallocate_memory_pop(population *pop)
{
	int i;
	for (i = 0; i < pop->size; i++)
	{
		deallocate_memory_ind(&(pop->ind[i]));
	}
	free(pop->ind);
	return;
}
