/*
 * mopso-d.c
 *
 *  Created on: 26/10/2010
 *      Author: Saúl Zapotecas
 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "global.h"
#include "rand.h"
#include "evaluate.h"
#include "moea_tools.h"
#include "update.h"

int compare_dist(const void *_a, const void *_b)
{
	distance_vector *a, *b;
	a = (distance_vector *) _a;
	b = (distance_vector *) _b;
	return (b->distance <= a->distance) ? 1 : 0;
}

void reinitialization_mechanism(double *x, double *v, double *pb, double *gb)
{
	int j;
	double m, s, xt;
	for (j = 0; j < mop.nreal; j++)
	{
		m = (gb[j] - pb[j]) / 2.0;
		s = fabs(gb[j] - pb[j]);
		xt = N(m, s);
		if (xt < mop.low_bound[j])
		{
			pb[j] = x[j] = mop.low_bound[j];
		}
		else if (xt > mop.up_bound[j])
		{
			pb[j] = x[j] = mop.up_bound[j];
		}
		else
		{
			pb[j] = x[j] = xt;
		}
		v[j] = 0.0;
	}
	return;
}

void update_particle_inertia(double *x, double *v, double *pb, double *gb)
{
	int j;
	double xt;
	double w = rndreal(0.1, 0.5);
	double c1 = rndreal(1.2, 2.0);
	double c2 = rndreal(1.2, 2.0);
	double r1 = rndreal(0.0, 1.0);
	double r2 = rndreal(0.0, 1.0);
	double gamma = 1.0;

	/*update_velocity(v, x, pb, gb);*/
	for (j = 0; j < mop.nreal; j++)
	{
		v[j] = w * v[j] + c1 * r1 * (pb[j] - x[j]) + c2 * r2 * (gb[j] - x[j]);
		xt = x[j] + v[j];
		if (xt < mop.low_bound[j])
		{
			x[j] = mop.low_bound[j];
			v[j] = -gamma * v[j];
		}
		else if (xt > mop.up_bound[j])
		{
			x[j] = mop.up_bound[j];
			v[j] = -gamma * v[j];
		}
		else
		{
			x[j] = xt;
		}
	}
	return;
}

void update_pbest(individual *ind, weight *W)
{
	double g1, g2;
	int i, index = ind->problem_id;

	g1 = objective(ind->obj, W->weight[index].w, moea.z); /* new particle */
	g2 = objective(ind->pb_obj, W->weight[index].w, moea.z); /* pbest 		*/
	if (ind->age >= moea.age || g1 <= g2)
	{
		ind->age = 0;
		for (i = 0; i < mop.nreal; i++)
		{
			ind->pb_xreal[i] = ind->xreal[i];
		}
		for (i = 0; i < mop.nobj; i++)
		{
			ind->pb_obj[i] = ind->obj[i];
		}
	}
	else
	{
		ind->age++;
	}
	return;
}

void next_pop(weight *W, population *gbest_pop, population *pop)
{
	int i;
	double *v, *x, *pb, *gb;
	int *vec_gbest = (int*) malloc(sizeof(int) * gbest_pop->size);
	for (i = 0; i < gbest_pop->size; i++)
	{
		vec_gbest[i] = i;
	}
	shuffle(vec_gbest, gbest_pop->size);
	for (i = 0; i < pop->size; i++)
	{
		v = pop->ind[i].vel;
		x = pop->ind[i].xreal;
		pb = pop->ind[i].pb_xreal;
		gb = gbest_pop->ind[vec_gbest[i]].xreal;

		if (pop->ind[i].age < moea.age)
		{
			update_particle_inertia(x, v, pb, gb);
		}
		else
		{
			reinitialization_mechanism(x, v, pb, gb);
		}
		evaluate_ind(&(pop->ind[i]));
		update_pbest(&(pop->ind[i]), W);
	}
	free(vec_gbest);
	printf("\tNext Cycle....... OK\n");
	return;
}

void get_gbest(weight *W, population *pop, population *gbest_pop)
{
	int i, j, index;
	int *v;
	double min, scalarization;

	if (gbest_pop->size != W->size)
	{
		printf("ERROR: Update, |gbest|!=|W|");
		exit(0);
	}
	v = (int*) malloc(sizeof(int) * pop->size);
	for (i = 0; i < pop->size; i++)
	{
		v[i] = -1;
	}
	for (j = 0; j < W->size; j++)
	{
		min = INF;
		index = 0;
		for (i = 0; i < pop->size; i++)
		{
			scalarization = objective(pop->ind[i].obj, W->weight[j].w, moea.z);
			if (scalarization < min && v[i] < 0)
			{
				min = scalarization;
				index = i;
			}
		}
		v[index] = 1;
		copy_ind(&(pop->ind[index]), &(gbest_pop->ind[j]));
		gbest_pop->ind[j].problem_id = j;
		gbest_pop->ind[j].scalarization = min;
	}
	free(v);
	printf("\tUpdating Swarm... OK\n");
	return;
}

