/*
 * moea_tools.c
 *
 *  Created on: 15/02/2010
 *      Author: Saúl Zapotecas
 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "global.h"
#include "rand.h"

void copy_ind(individual *ind1, individual *ind2)
{
	int i;

	for (i = 0; i < mop.nreal; i++)
	{
		ind2->xreal[i] = ind1->xreal[i];
		ind2->vel[i] = ind1->vel[i];
		ind2->pb_xreal[i] = ind1->pb_xreal[i];
	}
	for (i = 0; i < mop.nobj; i++)
	{
		ind2->obj[i] = ind1->obj[i];
		ind2->pb_obj[i] = ind1->pb_obj[i];
	}
	if (mop.nconst > 0)
	{
		for (i = 0; i < mop.nconst; i++)
		{
			ind2->constr[i] = ind1->constr[i];
		}
	}

	ind2->scalarization = ind1->scalarization;
	ind2->problem_id = ind1->problem_id;
	ind2->age = ind1->age;
	return;
}

void copy_pop(population *pop1, population *pop2)
{
	int i;
	if (pop1->size != pop2->size)
	{
		printf("ERROR: function copy_pop, different size\n");
	}
	for (i = 0; i < pop2->size; i++)
	{
		copy_ind(&(pop1->ind[i]), &(pop2->ind[i]));
	}
	return;
}

void shuffle(int *v, int size)
{
	int *a = NULL;
	int i;
	int rnd;
	int tmp;

	a = (int *) malloc(size * sizeof(int));

	for (i = 0; i < size; i++)
	{
		a[i] = v[i];
	}
	for (i = 0; i < size; i++)
	{
		rnd = rndint(i, size - 1);
		tmp = a[rnd];
		a[rnd] = a[i];
		v[i] = tmp;
	}
	free(a);
	return;
}

/* Routine to merge two populations into one */
void merge_population(population *pop1, population *pop2, population *pop3)
{
	int i, k;
	if (pop1->size + pop2->size != pop3->size)
	{
		printf("ERROR: Mixing population");
		exit(0);
	}
	for (i = 0; i < pop1->size; i++)
	{
		copy_ind(&(pop1->ind[i]), &(pop3->ind[i]));
	}
	for (i = 0, k = pop1->size; i < pop2->size; i++, k++)
	{
		copy_ind(&(pop2->ind[i]), &(pop3->ind[k]));
	}
	printf("\tMerge Swarm.... OK\n");
	return;
}

double euclidean_distance(double *x1, double *x2, int n)
{
	double res = 0.0;
	int i;
	for (i = 0; i < n; i++)
	{
		res += pow(x2[i] - x1[i], 2.0);
	}
	return sqrt(res);
}

void set_vector(double *v, double value, int n)
{
	int i;
	for (i = 0; i < n; i++)
	{
		v[i] = value;
	}
	return;
}

void copy_vector(double *a, double *b, int dim)
{
	int i;
	for (i = 0; i < dim; ++i)
	{
		b[i] = a[i];
	}
	return;
}
