/*
 * rand.c
 *
 *  Created on: 20/09/2009
 *      Author: Saúl Zapotecas
 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "global.h"
#include "rand.h"

double oldrand[55];
int jrand;

/* Get seed number for random and start it up */
void randomize()
{
	int j1;
	for (j1 = 0; j1 <= 54; j1++)
	{
		oldrand[j1] = 0.0;
	}
	jrand = 0;
	warmup_random(moea.seed);
	return;
}

/* Get randomize off and running */
void warmup_random(double seed)
{
	int j1, ii;
	double new_random, prev_random;
	oldrand[54] = seed;
	new_random = 0.000000001;
	prev_random = seed;
	for (j1 = 1; j1 <= 54; j1++)
	{
		ii = (21 * j1) % 54;
		oldrand[ii] = new_random;
		new_random = prev_random - new_random;
		if (new_random < 0.0)
		{
			new_random += 1.0;
		}
		prev_random = oldrand[ii];
	}
	advance_random();
	advance_random();
	advance_random();
	jrand = 0;
	return;
}

/* Create next batch of 55 random numbers */
void advance_random()
{
	int j1;
	double new_random;
	for (j1 = 0; j1 < 24; j1++)
	{
		new_random = oldrand[j1] - oldrand[j1 + 31];
		if (new_random < 0.0)
		{
			new_random = new_random + 1.0;
		}
		oldrand[j1] = new_random;
	}
	for (j1 = 24; j1 < 55; j1++)
	{
		new_random = oldrand[j1] - oldrand[j1 - 24];
		if (new_random < 0.0)
		{
			new_random = new_random + 1.0;
		}
		oldrand[j1] = new_random;
	}
}

/* Fetch a single random number between 0.0 and 1.0 */
double randomperc()
{
	jrand++;
	if (jrand >= 55)
	{
		jrand = 1;
		advance_random();
	}
	return ((double) oldrand[jrand]);
}

/* Fetch a single random integer between low and high including the bounds */
int rndint(int low, int high)
{
	int res;
	if (low >= high)
	{
		res = low;
	}
	else
	{
		res = low + (randomperc() * (high - low + 1));
		if (res > high)
		{
			res = high;
		}
	}
	return (res);
}

/* Fetch a single random real number between low and high including the bounds */
double rndreal(double low, double high)
{
	return (low + (high - low) * randomperc());
}

/* normal random variant generator */
double box_muller(double m, double s)
{
	/* mean m, standard deviation s */
	double x1, x2, w, y1;
	static double y2;
	static int use_last = 0;

	if (use_last) /* use value from previous call */
	{
		y1 = y2;
		use_last = 0;
	}
	else
	{
		do
		{
			x1 = 2.0 * randomperc() - 1.0;
			x2 = 2.0 * randomperc() - 1.0;
			w = x1 * x1 + x2 * x2;
		} while (w >= 1.0);

		w = sqrt((-2.0 * log(w)) / w);
		y1 = x1 * w;
		y2 = x2 * w;
		use_last = 1;
	}
	return (m + y1 * s);
}

/* normal random variant generator */
double N(double m, double sigma)
{
	return box_muller(m, sigma);
}
