/*
 * types.h
 *
 *  Created on: 10/02/2010
 *      Author: Saúl Zapotecas
 */

#ifndef TYPES_H_
#define TYPES_H_

/* ***************************************************
 * @xreal:		  Real variable
 * @obj:		  Objective values
 * @constr:		  Constraint values
 * ***************************************************/
typedef struct
{
	double *xreal;
	double *obj;
	double *constr;

	double *vel;
	double *pb_xreal;
	double *pb_obj;

	double scalarization;
	int age;
	int problem_id;
} individual;

/* ***************************************************
 * @ind:		  Individuals
 * @size:		  Population size
 * ***************************************************/
typedef struct
{
	individual *ind;
	int size;
} population;

/* ***************************************************
 * MOP parameters
 * 		   nreal: Number of decision variables
 * 		    nobj: Number of objectives
 *       nconstr: Number of constraints
 * 		up_bound: Array of up bound decision variables
 * 	   low_bound: Array of low bound decision variables
 * ***************************************************/
typedef struct
{
	int front_type;
	int nreal;
	int nobj;
	int nconst;
	double *up_bound;
	double *low_bound;
	char str_real_front_plot[255];
	char str_real_front_data[255];
	char str_mop_name[255];
} mop_parameters;

/* ***************************************************
 * MOEA parameters
 * 		pop_size: Population size
 * 		 max_gen: Maxim number iterations
 *			  Pc: Crossover probability
 *			  Pm: Mutation probability
 *	 evaluations: Number of evaluations of MOEA
 * ***************************************************/
typedef struct
{
	int pop_size;
	int max_gen;
	int age;
	int H;
	char str_moea_name[255];
	double *z;
	double *n;
	double Pc;
	double Pm;
	double seed;
	double distance;
	int evaluations;
} moea_parameters;

/* ***************************************************
 * 	new parameters
 * ***************************************************/
typedef struct
{
	double distance;
	int index;
} distance_vector;

typedef struct
{
	int *index;
	int size;
} neighbourhood;

typedef struct
{
	double *w;
	int aux;
} vector;

typedef struct
{
	vector *weight;
	int size;
} weight;

#endif /* TYPES_H_ */
